# Parameters

Param
(
    [parameter(mandatory)]
    [string]
    $azureSubscriptionID,

    [parameter(mandatory)]
	[string]
    $WindowsScriptName,

    [parameter(mandatory)]
	[string]
    $LinuxScriptName,

	[parameter(mandatory)]
	[string]
    $StorageResourceGroupName,

	[parameter(mandatory)]
    [string]
    $StorageAccountName,

	[parameter(mandatory)]
	[string]
    $StorageContainerName,

    [parameter(mandatory)]
	[string]
    $LinuxScriptURL
)

# Login As Automation Account

try
{
	$Conn = Get-AutomationConnection -Name AzureRunAsConnection
	Login-AzureRmAccount -ServicePrincipal -Tenant $Conn.TenantID `
	-ApplicationId $Conn.ApplicationID -CertificateThumbprint $Conn.CertificateThumbprint
	Write-Output "Successfully connected as Automation Account"
}
catch
{
    Write-Output "Error connecting as Automation Account"
    Write-Output $_.Exception.Message
}

$null = Set-AzureRMContext -SubscriptionId $azureSubscriptionID

# Initialization

Write-Output "Collecting all VMs"
$vms = @()
$vms = Get-AzureRmVM
$storagekey = (Get-AzureRmStorageAccountKey -Name $StorageAccountName -ResourceGroupName $StorageResourceGroupName).Key1
$PublicConfiguration = @{"fileUris" = @($LinuxScriptURL); "commandToExecute" = "./$LinuxScriptName"}
$PrivateConfiguration = @{"storageAccountName" = $StorageAccountName; "storageAccountKey" = $storagekey}

# Loop through each VM in the array and deploy the extension

foreach ($vm in $vms)
{
    $vmStatus = (Get-AzureRmVM -ResourceGroupName $vm.ResourceGroupName -Name $vm.Name -Status).Statuses.DisplayStatus[-1]

    Write-Output "Processing VM: $($vm.Name)"

    if ($vmStatus -ne 'VM running')
    {
        Write-Warning -Message "Skipping VM as it is not currently powered on"
    }
    else
    {
        #Check to see if Linux or Windows
        if ($vm.StorageProfile.OsDisk.OsType -eq 'Windows')
        {
            $extensions = Get-AzureRmVMExtension -ResourceGroupName $vm.ResourceGroupName -VMName $vm.Name -Name 'SophosAutoDeploy' -ErrorAction SilentlyContinue
            #Make sure the extension is not already installed before attempting to install it
            if (-not $extensions)
            {
                try 
                {
                    Write-Output "Adding SophosAutoDeploy extension to VM: $($vm.Name)"
                    Set-AzureRmVMCustomScriptExtension `
                    -ResourceGroupName $vm.ResourceGroupName `
                    -Location $vm.Location `
                    -VMName $vm.Name `
                    -Name "SophosAutoDeploy" `
                    -StorageAccountName $StorageAccountName `
                    -StorageAccountKey $storagekey `
                    -FileName $WindowsScriptName `
                    -ContainerName $StorageContainerName `
                    -Run $WindowsScriptName
                }
                catch 
                {
                    Write-Output "Failed to set custom script extension"
                    Write-Output $_.Exception.Message
                }
            }
            else
            {
                Write-Output "Skipping VM - Extension already installed"
            }
        }
        else
        {
            if ($vm.StorageProfile.OsDisk.OsType -eq 'Linux')
            {
                $extensions = Get-AzureRmVMExtension -ResourceGroupName $vm.ResourceGroupName -VMName $vm.Name -Name 'SophosAutoDeploy' -ErrorAction SilentlyContinue
                #Make sure the extension is not already installed before attempting to install it
                if (-not $extensions)
                {
                    try 
                    {
                        Write-Output "Adding SophosAutoDeploy extension to VM: $($vm.Name)"                 
                        Set-AzureRmVMExtension `
                        -ResourceGroupName $vm.ResourceGroupName `
                        -Location $vm.Location `
                        -VMName $vm.Name `
                        -Name "SophosAutoDeploy" `
                        -Publisher "Microsoft.Azure.Extensions" `
                        -ExtensionType "CustomScript" `
                        -TypeHandlerVersion "2.0" `
                        -Settings $PublicConfiguration `
                        -ProtectedSettings $PrivateConfiguration
                    }
                    catch 
                    {
                        Write-Output "Failed to set custom script extension"
                        Write-Output $_.Exception.Message
                    }
                }
                else
                {
                    Write-Output "Skipping VM - Extension already installed"
                }
        }
        }
    }
}